﻿//======================================================================================================================
// NunchuckVJoy project, version 1.0
// Giliam de Carpentier, Copyright (c) 2019.
// See decarpentier.nl/making-a-wireless-joystick for project details.
//======================================================================================================================
// Source code is licensed under the MIT license. 
// See mit.txt for details.
// Use at your own risk!
//======================================================================================================================

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NunchuckVJoy
{
    // This class to unpacks the data coming from a connected Bluetooth device and passes it on to the given vJoy device.
    public class BluetoothTranslator
    {
        public BluetoothTranslator(VJoystick vJoystick)
        {
            _vJoystick = vJoystick;
        }

        public void OnMessage(byte[] buffer)
        {
            // Expected data format:
            // Bit 0 - 6    : X axis
            // Bit 7        : 0 (NOT end of message)
            // Bit 8 - 14   : Y axis
            // Bit 15       : 0 (NOT end of message)
            // Bit 16 - 22  : Button states 0-6
            // Bit 23       : 0 (NOT end of message)
            // Bit 24 - 30  : Lower 7-bits of 8-bit CRC 
            // Bit 31       : 1 (end of message)

            // Check if the MSB values of the 4 bytes and the CRC value are correct
            if (buffer.Length == 4 &&
                ((buffer[0] | buffer[1] | buffer[2]) & 0x80) == 0x00 &&
                (CRC8.Calculate(0xAA, buffer,3) | 0x80) == buffer[3])
            {
                // Unpack the axis and button states
                float axisx = 2.0f * (((int)buffer[0] + (buffer[0] == 127 ? 1 : 0)) / 128.0f) - 1.0f;
                float axisy = 2.0f * (((int)buffer[1] + (buffer[1] == 127 ? 1 : 0)) / 128.0f) - 1.0f;
                bool button0 = (buffer[2] & 0x01) != 0;
                bool button1 = (buffer[2] & 0x02) != 0;

                // Send the new state to vJoy
                _vJoystick.Update(axisx, axisy, button0, button1);
            }
        }

        // Handle to the vJoy object
        private VJoystick _vJoystick;
    }
}
