﻿//======================================================================================================================
// NunchuckVJoy project, version 1.0
// Giliam de Carpentier, Copyright (c) 2019.
// See decarpentier.nl/making-a-wireless-joystick for project details.
//======================================================================================================================
// Source code is licensed under the MIT license. 
// See mit.txt for details.
// Use at your own risk!
//======================================================================================================================

namespace NunchuckVJoy
{
    // CRC8 contains helper functions to calculate an 8-bit CRC-8-ATM hash from a message.
    static class CRC8
    {
        // Calculate 8-bit CRC-8-ATM hash by starting with <prefix> and
        // integrating the first <count> bytes of <buffer>.
        public static byte Calculate(byte prefix, byte[] buffer, int count)
        {
            byte crc = prefix;
            byte polynomial = 0x07;
            for (int i = 0; i < count; ++i)
                crc = UpdatePoly(polynomial, crc, buffer[i]);
            return crc;
        }

        // Update the hash calculated so far, given the new data byte
        private static byte UpdatePoly(byte polynomial, byte crc, byte data)
        {
            crc = (byte)(crc ^ data);

            // Iterate over each bit of the CRC
            for (byte i = 0; i < 8; i++)
            {
                if ((crc & 0x80) != 0)
                {
                    crc <<= 1;
                    crc ^= polynomial;
                }
                else
                {
                    crc <<= 1;
                }
            }

            return crc;
        }
    }
}