﻿//======================================================================================================================
// NunchuckVJoy project, version 1.0
// Giliam de Carpentier, Copyright (c) 2019.
// See decarpentier.nl/making-a-wireless-joystick for project details.
//======================================================================================================================
// Source code is licensed under the MIT license. 
// See mit.txt for details.
// Use at your own risk!
//======================================================================================================================
// Requires the UwpDesktop package, which can be retrieved using NuGet as follows: 
// "Install-Package UwpDesktop -Version 10.0.14393.3"
//======================================================================================================================

using System;
using System.Threading;
using Windows.Devices.Bluetooth.GenericAttributeProfile;

namespace NunchuckVJoy
{
    // Entry point for the application, kicking off all necessary components.
    class Program
    {
        // Set up the necessary objects
        public Program()
        {
            // Create a joystick object for a virtual vJoy joystick with device id 1.
            _vJoystick = new VJoystick(1);

            // Prepare the translator to bridge between the vJoy joystick and the bluetooth connection
            _bluetoothTranslator = new BluetoothTranslator(_vJoystick);

            // Prepare to looking for a bluetooth connection
            _bluetoothConnection = new BluetoothConnection("GD3C@RP-", GattDeviceService.ConvertShortIdToUuid(0x0ffe1));
            _bluetoothConnection.OnConnect += (name) => Console.WriteLine("Connected to bluetooth device {0}", name);
            _bluetoothConnection.OnDisconnect += () => Console.WriteLine("Disconnected");
            _bluetoothConnection.OnMessage += _bluetoothTranslator.OnMessage;
        }

        // Start looking for vJoy and a bluetooth connection
        bool Start()
        {
            // Initialize the virtual joystick registered to vJoy
            if (!_vJoystick.Start())
                return false;
            
            // Start looking for a bluetooth connection
            _bluetoothConnection.Start();

            return true;
        }

        // Stop the program by killing any connection to a Bluetooth device and vJoy.
        void Stop()
        {
            _bluetoothConnection.Stop();
            _bluetoothConnection = null;

            _vJoystick.Stop();
            _vJoystick = null;
        }

        // Entry point called on application start.
        public static void Main(string[] args)
        {
            // Prepare a semaphore indicating we should exit when released. Nothing is currently triggering this.
            _exitSignal = new SemaphoreSlim(0, 1);

            var program = new Program();
            if (!program.Start())
                return;

            // Wait on release of exit semaphore
            _exitSignal.Wait();

            // Clean up the remains of the program
            program.Stop();
        }

        // Bluetooth connection instance
        private BluetoothConnection _bluetoothConnection;

        // Bluetooth data handler instance
        private BluetoothTranslator _bluetoothTranslator;

        // Handle to the vJoy application
        private VJoystick _vJoystick;

        // Semaphore keeping the application running.
        private static SemaphoreSlim _exitSignal;
    }
}